'use strict';

const domain = require('domain');
const { constructFromEncodedString } = require('@codefresh-io/authenticated-entity');
const { registerCorrelation } = require('./correlation');
const { REQUEST_ID_HEADER, AUTHENTICATED_ENTITY_HEADER, SOURCE_IP } = require('./headers');
const _ = require('lodash');

function _getIp(req) {

    const ip = req.headers['x-forwarded-for'] || req.connection.remoteAddress;

    return ip.split(',')[0];
}

module.exports = {
    newDomainMiddleware() {
        return (res, req, next) => {
            const dom = domain.create();

            const requestId = res.headers[REQUEST_ID_HEADER.toLowerCase()];
            const rawAuthenticatedEntity = res.headers[AUTHENTICATED_ENTITY_HEADER.toLowerCase()];
            const sourceIp = res.headers[SOURCE_IP.toLowerCase()] || _getIp(res);

            let authenticatedEntity = null;
            if (rawAuthenticatedEntity) {
                authenticatedEntity = constructFromEncodedString(rawAuthenticatedEntity);
            }

            dom.add(req);
            dom.add(res);

            dom.run(() => {
                try {
                    registerCorrelation(authenticatedEntity, requestId, sourceIp);
                    setImmediate(next);
                } catch (err) {
                    setImmediate(next, err);
                }
            });
        };
    }
};
