'use strict';

const chai = require('chai');
const sinonChai = require('sinon-chai');
const { System } = require('@codefresh-io/authenticated-entity');
const proxyquire = require('proxyquire');

const expect = chai.expect;
chai.use(sinonChai);


describe('Http infra unit test', () => {

    it('should throw error in case we don`t have a register domain', () => {
        const correlationProxy = proxyquire('../correlation', {
            'domain': {
                active: [],
            },
        });
        const authEntityToOverride = new System('microservice');
        expect(() => correlationProxy.setAuthenticatedEntity(authEntityToOverride))
            .to
            .throw(`Can't set authenticated entity. Domain isn't registered.`);

    });

    it('should throw error in case we already have an authenticated entity', () => {
        const correlationProxy = proxyquire('../correlation', {
            'domain': {
                active: [],
            },
        });
        const authEntityToOverride = new System('microservice');
        correlationProxy.registerCorrelation(authEntityToOverride);
        expect(() => correlationProxy.setAuthenticatedEntity(authEntityToOverride))
            .to
            .throw(`Can't set authenticated entity. Another entity is already set.`);

    });

    it('should not throw error in case we already have an authenticated entity and we pass allowOverride flag', () => {
        const correlationProxy = proxyquire('../correlation', {
            'domain': {
                active: [],
            },
        });
        const authEntityToOverride = new System('microservice');
        correlationProxy.registerCorrelation(authEntityToOverride);
        expect(() => correlationProxy.setAuthenticatedEntity(authEntityToOverride, true))
            .to
            .not
            .throw(`Can't set authenticated entity. Another entity is already set.`);

    });
    it('should return created source ip record from domain', () => {
        const correlationProxy = proxyquire('../correlation', {
            'domain': {
                active: [],
            },
        });
        const authEntityToOverride = new System('microservice');
        correlationProxy.registerCorrelation(authEntityToOverride, null, '127.0.0.1');
        expect(correlationProxy.getSourceIp())
            .eq('127.0.0.1');

    });

});
